		page    55,132
		Title	CMOS Clock Setting Program
		Subttl	Copyright (c) 1991  Rich Shealer, Jr.
;
; CMOSCLK  Version 1.0
;
; Rich Shealer, Jr.
; 333 East King Street
; York, PA  17403-5603
;
; (717) 843-1077
; 
; rshealer on BIX
;
;
; CMOSClk is a utility to store the Time and Date from an IBM AT or compatible
; into its CMOS.  Since DOS 3.3 and above already does this, it is not very
; useful unless the user is running DOS < 3.3.  However, I run into a lot of
; machines that run 3.1. So I wrote this program to use with Roedy Green's
; (Canadian Mind Products/roedy on BIX) CMOS(save/chk/rest) Programs.
; 
; If used with a autoexec.bat file the CMOS can be completely recovered on
; a system running any DOS without the user having to run setup.
;
; CMOSCLK.COM is to be freely distributed.
;   

; DOS Functions

DisplayString	equ	9
GetDate		equ	2ah
GetTime		equ	2ch
NormalExit	equ	4c00h


; CMOS Addresses


CMOSaddr	equ	70h		; Port that sets up hardware window
CMOSdata	equ	71h		; Data window for CMOS


CMOSSeconds	equ	0		; BCD Values
CMOSMinutes	equ	2
CMOSHours	equ	4
CMOSWeekDay	equ	6		; BCD Values
CMOSDay		equ	7
CMOSMonth	equ	8
CMOSYear	equ	9
CMOSCentury	equ	32h

; General

cr		equ	0dh
lf		equ	0ah
eof		equ	1ah

code		segment word public 'CODE'
		assume	cs:code,ds:code

		org	100h

main	proc	near

;	Date Section

;	This code takes the system date and stores it in the CMOS

start:  jmp     start1		; Bypass data segment (COM file)

SignOnMsg DB	'CMOSClk Version 1.0 - Store DOS System Clock to CMOS',cr,lf,lf
	DB      'This program has just taken the DOS system time and',cr,lf
	DB	'date, and stored it into the permanent CMOS RAM.',cr,lf,lf
	DB      'Copyright (c) 1991  Rich Shealer, Jr.',cr,lf
	DB	'This program may be freely distributed.',cr,lf,lf,'$',EOF	

start1:	mov	dx, offset SignOnMsg	; Display Signon Message
	mov	ah,DisplayString
	int	21h
 
	mov	ah,GetDate	; Get Date Function
	int	21h		; Call DOS
	
	mov	ah,al		; Perserve Day of Week
	cli			; Disable interupts

; Day of the Week
	mov     al,CMOSWeekDay	; We want to talk to the Day of the week
	out	CMOSaddr,al	; Set up proper port
	jmp	$+2		; Pause because of hardware limits

	mov     al,ah		; Restore Day of Week to AL
	out	CMOSdata,al	; Store day of week to CMOS
	jmp	$+2		; Pause because of hardware limits

; Day
	mov     al,CMOSDay	; We want to talk to the Day
	out	CMOSaddr,al	; Set up proper port
	jmp	$+2		; Pause because of hardware limits

	mov	al,dl		; Move day off the month to AL
	call	Int2BCD		; Convert it to BCD
	out	CMOSdata,al	; Store day to CMOS
	jmp	$+2		; Pause because of hardware limits

; Month
	mov	al,CMOSMonth	; Set up request for month
	out	CMOSaddr,al	; Set up port window
	jmp	$+2		; Pause because of hardware limits

	mov	al,dh		; Move month to AL
	call	Int2BCD		; Convert it to BCD
	out	CMOSdata,al	; Store Month to CMOS
	jmp	$+2		; Pause because of hardware limits

; Year
	mov	al,CMOSYear	; Set up request for Year
	out	CMOSaddr,al	; Set up port window
	jmp	$+2		; Pause because of hardware limits
	mov 	bl,19		; BX+19=century
	sub	cx,1900		; We want the xx part of 19xx

TestCentury:
	cmp	cx,100		; year > 2000?
	jl	dateok
	sub     cx,100		; Adjust Century
	inc	bx
	jmp	TestCentury

dateok: mov	al,cl		; Move year to AL
	call	Int2BCD		; Convert it to BCD
	out	CMOSdata,al	; Store Year to CMOS
	jmp	$+2		; Pause because of hardware limits

	mov	al,CMOSCentury
	out	CMOSaddr,al	; Set up port window
	jmp	$+2		; Pause because of hardware limits
	mov	al,bl		; Move Century to AL
	call	Int2BCD		; Convert it to BCD
	out	CMOSdata,al	; Store Century to CMOS
	jmp	$+2		; Pause because of hardware limits

	sti			; enable interupts

;	This section takes the current time and stores it in CMOS

	mov	ah,GetTime
	int	21h

	cli			; Hold all calls
; seconds
	mov     al,CMOSSeconds	; Set up CMOS window for seconds
	out	CMOSaddr,al	; Set up proper port
	jmp	$+2		; Pause because of hardware limits

	mov     al,DH		; Get Seconds
	call    int2BCD		; Convert value to BCD
	out	CMOSdata,al	; Store Seconds to CMOS
	jmp	$+2		; Pause because of hardware limits

; minutes
	mov     al,CMOSMinutes	; Set up CMOS window for minutes
	out	CMOSaddr,al	; Set up proper port
	jmp	$+2		; Pause because of hardware limits

	mov     al,CL		; Get Minutes
	call    int2BCD		; Convert value to BCD
	out	CMOSdata,al	; Store Minutes CMOS
	jmp	$+2		; Pause because of hardware limits

; hours
	mov     al,CMOSHours	; Set up CMOS window for Hours
	out	CMOSaddr,al	; Set up proper port
	jmp	$+2		; Pause because of hardware limits

	mov     al,CH		; Get Hours
	call    int2BCD		; Convert value to BCD
	out	CMOSdata,al	; Store Hours CMOS
	jmp	$+2		; Pause because of hardware limits

	sti			; re-enable interupts

	mov	ax,NormalExit	; Exit normaly RC = 0
	int	21h

main	endp

int2BCD	proc

;	This routine takes the integer in AL and converts it to BCD
;
;	Entry:
;		AL = Value to convert
;	Exit
;		AL = BCD
;	Trashes:
;		AH   

	mov	ah,0		; Clear our result
CountTens:
	sub	al,10		; How many tens do we have?
	jc	AddOnes		; loop until all tens gone
	inc	ah		; Add one to tens column 
	jmp	CountTens

AddOnes:
	add	al,10		; Add back the extra ten
	rol	ah,1		; Shift Tens Column Left
	rol	ah,1
	rol	ah,1
	rol	ah,1

	or	al,ah		; al = BCD Value

	ret

int2BCD endp

code		ends

; stack   segment stack           ; Dummy Stack
; stack   ends

		end	start
